setwd("path")

library(dplyr)
library(tidyverse)
library(lme4qtl)
library(kinship2)
library(bdsmatrix)
library(lqmm)
library(ald)

#Loading Data
data<-read.table("data_sardinian.csv", header=T, sep=";")
data<-data[order(data$SUBFAMID),]

#Calculating kinship matrix
data<-data[order(data$SUBFAMID),]
kped<-with(data, pedigree(id=data$id, dadid=data$father_id, momid=data$mother_id, sex=data$sex, famid=data$FAMID))
kmat <- kinship(kped)
kmat<-2*kmat
kmat<-as.matrix(kmat);

#Calculating household matrix
t<-table(data$SUBFAMID)
r<-sum(t^2)
id <-data$id
hmat<- as.matrix(bdsmatrix(blocksize=t, block=rep(1,r), dimnames=list(id,id)))

#Excluding subjects without protein levels and without age at blood sampling
data<-data[which(!is.na(data$arg2_hpa000663)),]
data<-data[which(!is.na(data$Age)),]


bb<-data$SUBFAMID
bb<-bb[!duplicated(bb)]
nfam<-length(bb)

#LMM power analysis

pval<-NULL
for(y in 1:2000){
  data$ms<-rbinom(206,1,0.3254717)
  ai<-NULL
  ci<-NULL
  ei<-NULL
  for(i in 1:nfam){
    fam <- which(data$SUBFAMID==bb[i])
    ni <- length(fam)			# number of family members for family i
    Ki <- kmat[fam,fam]
    Hi <- hmat[fam,fam]
    
    #Kinship and shared environment effects
    if(length(fam)>1){
      kkl<-SimDesign::rmvnorm(n=1, mean=rep(0, length(fam)), sigma=0.15*Ki) #h2=0.1
      ai<-c(ai,t(kkl) )                
      kkl2<-SimDesign::rmvnorm(n=1, mean=rep(0, length(fam)), sigma=0.10*Hi) #h2=0.1
      ci<-c(ci,t(kkl2)) 
      kkl3<-SimDesign::rmvnorm(n=1, mean=rep(0, length(fam)), sigma=(1-0.25-var(data$ms[fam]*0.5 + data$sex[fam]*0.05 + data$Age[fam]*0.01))*diag(ni)) #h2=0.1
      ei<-c(ei,t(kkl3)) 
    }
    if(length(fam)==1){
      kkl<-rnorm(n=1, mean=0, sd=sqrt(0.15)) #h2=0.1
      ai<-c(ai,kkl )                
      kkl2<-rnorm(n=1, mean=0, sd=sqrt(0.10)) #h2=0.1
      ci<-c(ci,kkl2 )   
      kkl3<-rnorm(n=1, mean=0, sd=sqrt(1-0.25)) #h2=0.1
      ei<-c(ei,kkl3 )  
    }
    i<-i+1
  }
  data$ai<-ai
  data$ci<-ci
  data$ei<-ei
  
  #Fixed effects
  data <- data %>%
    mutate(prot =ifelse(ms==1,ms*0.5 + sex*0.05+ Age*0.01+ ai + ci + ei, sex*0.05 +ai + ci + ei))
  
  
  model <- as.formula(" prot ~ (1|id) + (1|SUBFAMID)+ms+sex+Age")
  r <- relmatLmer(model, data=data,REML=F, relmat = list(id = kmat))
  r<-summary(r)
  pval<-rbind(pval,signif(1 - pchisq((r$coefficients[2,1]/r$coefficients[2,2])^2, 1), 7))
  #Power
  print(sum(table(pval[pval<0.05]))/nrow(pval))
  
}

#LQMM power analysis

pval2<-NULL
for(y in 1:2000){
  data$ms<-rbinom(206,1,0.3254717)
  ai<-NULL
  ci<-NULL
  ei<-NULL
  for(i in 1:nfam){
    fam <- which(data$SUBFAMID==bb[i])
    ni <- length(fam)			# number of family members for family i
    Ki <- kmat[fam,fam]
    Hi <- hmat[fam,fam]
    
    #Kinship and shared environment effects
    if(length(fam)>1){
      kkl<-SimDesign::rmvnorm(n=1, mean=rep(0, length(fam)), sigma=0.15*Ki) #h2=0.1
      ai<-c(ai,t(kkl) )                
      kkl2<-SimDesign::rmvnorm(n=1, mean=rep(0, length(fam)), sigma=0.10*Hi) #h2=0.1
      ci<-c(ci,t(kkl2)) 
      kkl3<-rALD(length(fam), 0 , sigma=0.2, p=0.5) #h2=0.1
      
      ei<-c(ei,rep(t(kkl3))) 
    }
    if(length(fam)==1){
      kkl<-rnorm(n=1, mean=0, sd=sqrt(0.15)) #h2=0.1
      ai<-c(ai,kkl )                
      kkl2<-rnorm(n=1, mean=0, sd=sqrt(0.10)) #h2=0.1
      ci<-c(ci,kkl2 )   
      kkl3<-rALD(n=1, 0 , sigma=0.2, p=0.5) #h2=0.1
      ei<-c(ei,kkl3 )  
    }
  }
  data$ai<-ai
  data$ci<-ci
  data$ei<-ei
  
  data<-data[order(data$SUBFAMID),]
  
  km<-chol(kmat)
  
  ccc<-rep(1,206)
  Zt2 <- km[as.character(data$id), as.character(data$id)] %*% ccc
  
  #Fixed effects
  data <- data %>%
    mutate(prot =ms*0.5 + sex*0.05+ Age*0.01 + ai + ci + ei)
  
  model <- as.formula(" prot ~ Age+sex+ms")
  
  rqr<-lqmm(fixed=model, random = ~ Zt2, cov="pdDiag", group=SUBFAMID, 
            type="robust", data=data, na.action=na.omit, tau=c(0.5),  control = lqmmControl(method = "gs", LP_max_iter=5000))
  
  
  pval2<-rbind(pval2,summary(lqmm::boot(rqr, R = 50, startQR = FALSE))[4,5])
  #Power
  print(sum(table(pval2[pval2<0.05]))/nrow(pval2))
  
}
