# forward.lmer: a function for stepwise regression using lmer mixed effects models
# Author: Rense Nieuwenhuis
forward.lmer <- function(
  start.model, blocks,
  max.iter=1, sig.level=FALSE,
  zt=FALSE, print.log=TRUE)
{
  
  # Initialysing internal variables
  log.step <- 0
  log.LL <- log.p <- log.block <- zt.temp <- log.zt <- NA
  model.basis <- start.model
  
  # Maximum number of iterations cannot exceed number of blocks
  if (max.iter > length(blocks)) max.iter <- length(blocks)
  
  # Setting up the outer loop
  for(i in 1:max.iter)
  {
    
    models <- list()
    
    # Iteratively updating the model with addition of one block of variable(s)
    # Also: extracting the loglikelihood of each estimated model
    for(j in 1:length(blocks))
    {
      models[[j]] <- update(model.basis, as.formula(paste(". ~ . + ", blocks[j])))
    }
    
    LL <- unlist(lapply(models, logLik))
    
    # Ordering the models based on their loglikelihood.
    # Additional selection criteria apply
    for (j in order(LL, decreasing=TRUE))
    {
      
      
      ##############
      ############## Selection based significance of added variable-block
      ##############
      
      if(zt != FALSE)
      {
        b.model <- summary(models[[j]])$coefficients
        diff.par <- setdiff(rownames(b.model), rownames(summary(model.basis)$coefficients))
        if (length(diff.par)==0) break
        sig.par <- FALSE
        
        for (k in 1:length(diff.par))
        {
          if(abs(b.model[which(rownames(b.model)==diff.par[k]),3]) > zt)
          {
            sig.par <- TRUE
            zt.temp <- b.model[which(rownames(b.model)==diff.par[k]),3]
            break
          }
        }
        
        if(sig.par==TRUE)
        {
          model.basis <- models[[j]]
          
          # Writing the logs
          log.step <- log.step + 1
          log.block[log.step] <- blocks[j]
          log.LL[log.step] <- as.numeric(logLik(model.basis))
          log.zt[log.step] <- zt.temp
          blocks <- blocks[-j]
          
          break
        }
      }
    }
  }
  
  ## Create and print log
  log.df <- data.frame(log.step=1:log.step, log.block, log.LL, log.p, log.zt)
  if(print.log == TRUE) print(log.df, digits=4)
  
  ## Return the 'best' fitting model
  return(model.basis)
}

"%nin%"<-Negate("%in%")

setwd("path")

#Loading Data
data<-read.table("data_sardinian.csv", header=T, sep=";")

#Calculating kinship matrix
data<-data[order(data$SUBFAMID),]
kped<-with(data, pedigree(id=data$id, dadid=data$father_id, momid=data$mother_id, sex=data$sex, famid=data$SUBFAMID))
kmat <- kinship(kped)
kmat<-2*kmat
kmat<-as.matrix(kmat);


#Excluding subjects without protein levels and without age at blood sampling
data<-data[which(!is.na(data$arg2_hpa000663)),]
data<-data[which(!is.na(data$Age)),]

#Protein of interest and associatd SNPs with p<0.10
name<-"c9_hpa029577"
snps<-c("imm_10_6139051",
           "X1kg_3_27766618",
           "imm_18_65694127",
           "rs12460421",
           "rs3007421",
           "imm_17_37749689",
           "rs4410871",
           "rs4468527",
           "imm_20_44167717",
           "rs17470892",
           "imm_14_68335317",
           "imm_1_199142520",
           "imm_11_127916046"
)

name<-"cyp24a1_hpa022261"
snps<-c("rs1870071",
           "imm_5_35893461",
           "imm_12_6321206",
           "rs7196953",
           "X1kg_1_93201835",
           "imm_17_37749689",
           "X1kg_1_92932826",
           "imm_10_64191835",
           "rs7665090",
           "imm_5_55476487",
           "rs4609871",
           "rs12460421"
)

name<-"sst_hpa019472"
snps<-c("rs2413436",
           "imm_7_50304754",
           "rs4468527",
           "rs7216796",
           "rs6072343",
           "rs2283792"
)

name<-"cp_hpa001834"
snps<-c("rs2082881",
           "imm_5_35893461",
           "rs756699",
           "X1kg_8_79720399",
           "rs7216796",
           "imm_6_138012651",
           "imm_17_37749689",
           "rs137956",
           "rs6072343",
           "rs2445610",
           "imm_1_158987178"
)

name<-"plat_hpa003412"
snps<-c("imm_1_158987178",
           "imm_5_35893461",
           "rs12086448",
           "imm_2_61029577",
           "imm_6_159394612",
           "rs9846396",
           "rs8107548",
           "rs2082881",
           "X1kg_8_79720399",
           "rs3007421",
           "rs4728142",
           "rs4410871",
           "rs9947399",
           "rs12695416"
)

# The code below is adapted for C9 protein, it should be updated where specified if protein is changed

snppp<-paste(snps, collapse = "+")

#Standardizing protein levels
data_controls<-data[data$ms==0 & !is.na(data$imm_10_6139051),]
sd<-sd(data_controls[,as.character(name)])
meancontr<-mean(data_controls[,as.character(name)])
data_controls[,as.character(name)]<-(data_controls[,as.character(name)]-meancontr)/sd


data_controls<-data_controls[order(data_controls$id),]
kmat2<-kmat[as.character(data_controls$id), as.character(data_controls$id)]
#Null model to start stepwise procedure
model2 <- as.formula(paste(name, " ~ (1|id) + (1|SUBFAMID)+ sex+Age", collapse = " "))
m1 <- relmatLmer(model2, data=data_controls,REML=F, relmat = list(id = kmat2))

#Stepwise procedure which includes SNPs at 0.05 level
m1<-summary(forward.lmer(m1, blocks=snps, max.iter=50, zt=1.96))

pvalues<-signif(1 - pchisq((m1$coefficients[,1]/m1$coefficients[,2])^2, 1), 10)
significant_snps<-names(pvalues[which(pvalues<0.01 & names(pvalues) %nin% c("(Intercept)", "Age", "sex"))])

beta<-m1$coefficients[,1]

#Estimating marginal proportion of protein level variability explained by significant SNPs

#Sex and Age component
XBagesex<-  beta[2] * data_controls$sex+ beta[3] * data_controls$Age


which(pvalues<0.01 & names(pvalues) %nin% c("(Intercept)", "Age", "sex"))
#Significant SNPs (to be updated if protein is changed)
sig_X1<-beta[4] * data_controls[,as.character(names(beta)[4])]+
  beta[5] * data_controls[,as.character(names(beta)[5])]


which(pvalues>0.01 & names(pvalues) %nin% c("(Intercept)", "Age", "sex"))
#Not significant SNPs component to be updated if protein is changed)
notsig_X1<- beta[6] * data_controls[,as.character(names(beta)[6])]+
  beta[7] * data_controls[,as.character(names(beta)[7])]+
  beta[8] * data_controls[,as.character(names(beta)[8])]

#All covariates component
XB<-sig_X1+XBagesex+ notsig_X1

#Significant SNPs and other covariates covariance
cov<-cov(sig_X1
         , XBagesex+
           notsig_X1)

#heritability component
a<-as.data.frame(m1[["varcor"]][["id"]])
#shared environment component
b<-as.data.frame(m1[["varcor"]][["SUBFAMID"]])

#Marginal proportion of protein level variability explained by significant SNPs
R2SNPs<-(var(sig_X1)+cov) /( m1$sigma^2 + var(XB) + a$`(Intercept)`+b$`(Intercept)`)



BCA2<-function(boot){
  #BCA method
  #Desired quantiles
  alpha<-0.05
  u <- c(alpha/2, 1-alpha/2) 
  
  #Compute constants
  z0 <- qnorm(mean(boot <= R2SNPs))
  zu <- qnorm(u)
  
  group_unique <- unique(data_controls$SUBFAMID)
  
  n<-length(group_unique)
  #n<-nrow(data2)
  I <- rep(NA, n)
  jack <- rep(NA, n)
  i<-1
  for(i in 1:n){
    #Remove ith data point
    xnew <- data_controls[-which(data_controls$SUBFAMID==group_unique[i]),]
    #xnew <- data[-i,]
    kmat2<-kmat[as.character(xnew$id), as.character(xnew$id)]
    #Estimate theta
    
    
    model2 <- as.formula(paste(name, " ~ (1|id) + (1|SUBFAMID) + sex + Age+",snppp, collapse = " "))
    

    m1 <- relmatLmer(model2, data=xnew,REML=F, relmat = list(id = kmat2))
    m1<-summary(m1)
    beta<-m1$coefficients[,1]
    
    #Estimating marginal proportion of protein level variability explained by significant SNPs
    
    #Sex and Age component
    XBagesex<-  beta[2] * xnew$sex+ beta[3] * xnew$Age
    
    #Significant SNPs (to be updated if protein is changed)
    sig_X1<-beta[4] * xnew[,as.character(names(beta)[4])]+
      beta[5] * xnew[,as.character(names(beta)[5])]
    
    #Not significant SNPs component (to be updated if protein is changed)
    notsig_X1<- beta[6] * xnew[,as.character(names(beta)[6])]+
      beta[7] * xnew[,as.character(names(beta)[7])]+
      beta[8] * xnew[,as.character(names(beta)[8])]
    
    #All covariates component
    XB<-sig_X1+XBagesex+ notsig_X1
    
    #Significant SNPs and other covariates covariance
    cov<-cov(sig_X1
             , XBagesex+
               notsig_X1)
    #heritability component
    a<-as.data.frame(m1[["varcor"]][["id"]])
    #shared environment component
    b<-as.data.frame(m1[["varcor"]][["SUBFAMID"]])
    VXF<-(var(sig_X1)+cov) /( m1$sigma^2 + var(XB) + a$`(Intercept)`+b$`(Intercept)`)
    
    I[i] <- (n-1)*(R2SNPs - VXF)
    jack[i]<-VXF
  }
  #Estimate a
  a <- (sum(I^3)/sum(I^2)^1.5)/6
  
  acc.num <- sum((mean(jack)-jack)^3)
  
  acc.denom <- sum((mean(jack)-jack)^2)
  
  accelerate <- acc.num/(6*acc.denom^1.5)
  #Adjusted quantiles
  u_adjusted <- pnorm(z0 + (z0+zu)/(1-a*(z0+zu))) 
  
  #Accelerated Bootstrap CI
  Bca<-quantile(boot, u_adjusted)
  return(Bca)
}

"nendsWith"<-Negate("endsWith")
substrRight <- function(x, n){
  substr(x, nchar(x)-n+1, nchar(x))
}
VXFboot<- vector(mode = "list")
b<-data_controls
b<-unique(b$SUBFAMID)
n<-length(b)
#Number of bootstrap replications
B<-1000
Esti<-numeric(B)
set.seed(1059)
for(i in 1:1000){
  index <-sample(b,
                 size=n,
                 replace=TRUE) #indicatore per campionare
  index<-as.data.frame(index)
  colnames(index)<-"SUBFAMID"
  
  tot3<-merge(data_controls, index, by="SUBFAMID", all.y=T, all.x=F)
  kmat2<-kmat[as.character(tot3$id),as.character(tot3$id)]
  
  
  c1.rle <- rle(tot3$id)
  tot3$id2 <- paste0(rep(c1.rle$values, times = c1.rle$lengths), "_",
                     unlist(lapply(c1.rle$lengths, seq_len)))
  
  colnames(kmat2)<-tot3$id2
  rownames(kmat2)<-tot3$id2
  
  kmat2[kmat2 ==1]<-0
  diag(kmat2)<-1
  
  kmat2[which(endsWith(rownames(kmat2),"_1")),which(nendsWith(colnames(kmat2), "_1"))]<-0
  kmat2[which(endsWith(rownames(kmat2),"_2")),which(nendsWith(colnames(kmat2), "_2"))]<-0
  kmat2[which(endsWith(rownames(kmat2),"_3")),which(nendsWith(colnames(kmat2), "_3"))]<-0
  kmat2[which(endsWith(rownames(kmat2),"_4")),which(nendsWith(colnames(kmat2), "_4"))]<-0
  kmat2[which(endsWith(rownames(kmat2),"_5")),which(nendsWith(colnames(kmat2), "_5"))]<-0
  kmat2[which(endsWith(rownames(kmat2),"_6")),which(nendsWith(colnames(kmat2), "_6"))]<-0
  kmat2[which(endsWith(rownames(kmat2),"_7")),which(nendsWith(colnames(kmat2), "_7"))]<-0
  kmat2[which(endsWith(rownames(kmat2),"_8")),which(nendsWith(colnames(kmat2), "_8"))]<-0
  kmat2[which(endsWith(rownames(kmat2),"_9")),which(nendsWith(colnames(kmat2), "_9"))]<-0
  kmat2[which(endsWith(rownames(kmat2),"_10")),which(nendsWith(colnames(kmat2), "_10"))]<-0
  
  tot3$SUBFAMID2<- paste(as.character(tot3$SUBFAMID), substrRight(tot3$id2, 2), sep = "")
  #tot3<-tot3[order(tot3$SUBFAMID2),]
  
  model2 <- as.formula(paste(name, " ~ (1|id2) + (1|SUBFAMID2) + sex +Age+",snppp, collapse = " "))
  
  m1 <- relmatLmer(model2, data=tot3,REML=F, relmat = list(id2 = kmat2))
  m1<-summary(m1)
  beta<-m1$coefficients[,1]
  #Estimating marginal proportion of protein level variability explained by significant SNPs
  
  #Sex and Age component 
  XBagesex<-  beta[2] * tot3$sex+ beta[3] * tot3$Age
  
  #Significant SNPs (to be updated if protein is changed)
  sig_X1<-beta[4] * tot3[,as.character(names(beta)[4])]+
    beta[5] * tot3[,as.character(names(beta)[5])]
  
  #Not significant SNPs component (to be updated if protein is changed)
  notsig_X1<- beta[6] * tot3[,as.character(names(beta)[6])]+
    beta[7] * tot3[,as.character(names(beta)[7])]+
    beta[8] * tot3[,as.character(names(beta)[8])]
  
  #All covariates component
  XB<-sig_X1+XBagesex+ notsig_X1
  
  #Significant SNPs and other covariates covariance
  cov<-cov(sig_X1
           , XBagesex+
             notsig_X1)
  a<-as.data.frame(m1[["varcor"]][["id2"]])
  a2<-as.data.frame(m1[["varcor"]][["SUBFAMID2"]])
  
  VXF<-(var(sig_X1)+cov) /( m1$sigma^2 + var(XB) + a$`(Intercept)`+a2$`(Intercept)`)
  

  VXFboot[i]<-VXF
  print(i)
}
boot<-unlist(VXFboot)
R2BcaCI<-BCA2(boot)
R2SNPs
R2BcaCI
